# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winrt._winrt_windows_networking_servicediscovery_dnssd import (
    DnssdRegistrationResult,
    DnssdServiceInstance,
    DnssdServiceInstanceCollection,
    DnssdServiceWatcher,
)

__all__ = [
    "DnssdRegistrationStatus",
    "DnssdServiceWatcherStatus",
    "DnssdRegistrationResult",
    "DnssdServiceInstance",
    "DnssdServiceInstanceCollection",
    "DnssdServiceWatcher",
]

class DnssdRegistrationStatus(enum.IntEnum):
    SUCCESS = 0
    INVALID_SERVICE_NAME = 1
    SERVER_ERROR = 2
    SECURITY_ERROR = 3

class DnssdServiceWatcherStatus(enum.IntEnum):
    CREATED = 0
    STARTED = 1
    ENUMERATION_COMPLETED = 2
    STOPPING = 3
    STOPPED = 4
    ABORTED = 5

winrt.system._mixin_sequence(DnssdServiceInstanceCollection)
